/* Fichier "tesselation.c"
 * fonctions qui transforment le tableau de hauteurs en display list
 * pour OpenGL
 * 
 * ATTENTION: ces fonctions doivent etre appeles APRES INITIALISATION
 *  d'OpenGL
 */

#include "projet.h"

/* pour sauter une arrete dans le mode fil de fer */
static int edge_flag = 1; 

/* ======================================================
 * tessallate_terrain
 *
 * Description: cree la display list associee a un terrain,
 *  i.e. transforme le tableau de hauteurs en display list pour OpenGL
 *
 * Parametres: pointeur sur le terrain  traiter 
 * =====================================================
 */
void
tessallate_terrain (TER_terrain_t * terrain)
{
  int i, j;
  float x, z;
  float inc;
  vector3d_t v[3];
  triangle3d_t triangle;
  int num_triangles = 0;

  glDeleteLists (terrain->displaylist, 1);
  terrain->displaylist = glGenLists (1);
  glNewList (terrain->displaylist, GL_COMPILE);

  /* quelle que soit la valeur de terrain->size, on effectue un rendu
     de taille 128x128 pour OpenGL.
     Par consquent, le parametre terrain->size represente la precision
     du rendu */
  inc = 128 / (float) terrain->size;

  glBegin (GL_TRIANGLES);

  /* le terrain est centr sur l'origine, dessin entre -64 et +64 */
  z = -64.0f;
  for (j = 0; j < terrain->size - 1; j++) {
    x = -64.0f;
    for (i = 0; i < terrain->size - 1; i++) {

      /* decoupage de chaque carr en 2 triangles */
      set_vector (&v[0], x + inc, TER_get_heightmap_value (terrain, i + 1, j), z);
      set_vector (&v[1], x, TER_get_heightmap_value (terrain, i, j), z);
      set_vector (&v[2], x, TER_get_heightmap_value (terrain, i, j + 1), z + inc);

      set_triangle (&triangle, v[0], v[1], v[2]);
      draw_triangle (triangle, terrain->color_scheme);

      set_vector (&v[0], x + inc, TER_get_heightmap_value (terrain, i + 1, j), z);
      set_vector (&v[1], x, TER_get_heightmap_value (terrain, i, j + 1), z + inc);
      set_vector (&v[2], x + inc, TER_get_heightmap_value (terrain, i + 1, j + 1),
		  z + inc);

      set_triangle (&triangle, v[0], v[1], v[2]);
      draw_triangle (triangle, terrain->color_scheme);

      num_triangles += 2;

      x += inc;
    }
    z += inc;
  }
  glEnd ();

  /* create_skybox(terrain); */

  printf ("TESSELLATION: %d triangles\n", num_triangles);

  glEndList ();
  terrain->generated = 1;
}

/* ======================================================
 * draw_triangle
 *
 * Description: dessine un triangle et lui assigne une couleur en
 *  fonction de son altitude
 * Parametres: le triangle  dessiner 
 * =====================================================
 */
void
draw_triangle (triangle3d_t triangle, int colorscheme)
{
  float *coord1, *coord2, *coord3;

  /*GLfloat mat_specular[] = { 1.0, 1.0, 1.0, 1.0 }; */
  GLfloat mat_specular[] = { 0.4, 0.4, 0.4, 0.4 };
  GLfloat mat_shininess[] = { 3.0 };
  GLfloat mat_amb_diff[] = { 1.0, 1.0, 1.0, 1.0 };

  glMaterialfv (GL_FRONT, GL_SPECULAR, mat_specular);
  glMaterialfv (GL_FRONT, GL_SHININESS, mat_shininess);
  glMaterialfv (GL_FRONT, GL_AMBIENT_AND_DIFFUSE, mat_amb_diff);

  coord1 = triangle.vertices[0].coord;
  coord2 = triangle.vertices[1].coord;
  coord3 = triangle.vertices[2].coord;

  glNormal3fv (triangle.normal.coord);

  /* on place les 3 sommets */
  if (edge_flag)
    glEdgeFlag (1);
  assign_color (mat_amb_diff, coord1[1], colorscheme);
  glMaterialfv (GL_FRONT, GL_AMBIENT_AND_DIFFUSE, mat_amb_diff);
  glVertex3fv (coord1);

  if (!edge_flag)
    glEdgeFlag (1);
  assign_color (mat_amb_diff, coord2[1], colorscheme);
  glMaterialfv (GL_FRONT, GL_AMBIENT_AND_DIFFUSE, mat_amb_diff);
  glVertex3fv (coord2);

  if (edge_flag)
    glEdgeFlag (0);
  assign_color (mat_amb_diff, coord3[1], colorscheme);
  glMaterialfv (GL_FRONT, GL_AMBIENT_AND_DIFFUSE, mat_amb_diff);
  glVertex3fv (coord3);

  edge_flag = !edge_flag;
}

/* ======================================================
 * create_test_square
 *
 * Description: permet de tester le rendu quand rien ne marche :-)
 *  cre une display list avec un terrain plat dedans
 * Parametres: pointeur sur terrain_t 
 * =====================================================
 */
void
create_test_square (TER_terrain_t * terrain)
{

  vector3d_t v[3];
  triangle3d_t triangle;

  terrain->displaylist = glGenLists (1);
  glNewList (terrain->displaylist, GL_COMPILE);

  glBegin (GL_TRIANGLES);

  set_vector (&v[0], -64, 0, -64);
  set_vector (&v[1], -64, 0, 64);
  set_vector (&v[2], 64, 0, 64);

  set_triangle (&triangle, v[0], v[1], v[2]);
  draw_triangle (triangle, terrain->color_scheme);

  set_vector (&v[0], -64, 0, -64);
  set_vector (&v[1], 64, 0, 64);
  set_vector (&v[2], 64, 0, -64);

  set_triangle (&triangle, v[0], v[1], v[2]);
  draw_triangle (triangle, terrain->color_scheme);

  glEnd ();

  glEndList ();
}
